/*
 * Program to perform "edits" required for two major changes upcoming for
 * linux 2.5.  The changes are:
 *
 * 1) Move the responsibility for locking down from the mid-level to each
 *    low-level driver.  In addition, eliminate usage of io_request_lock
 *    at the same time.  In general, locking should be handled in an identical
 *    fashion as in linux 2.4.0, main difference is that finer grained locks
 *    are used.  Caution: the NCR53c9x based drivers all share a common lock
 *    because of the way the drivers are built.
 *
 *    The changes are in several areas:
 *        a) Wrapper functions are generated and injected to handle the
 *           new responsibilities.  These are generally injected just
 *           prior to the function being wrapped.
 *        b) A lock definition is injected, typically just after the
 *           include of the main .h file associated with the driver.
 *        c) Prototypes for the wrapper functions are injected, typically
 *           just prior to the prototypes for the functions being wrapped.
 *        d) The template initializer is updated to use the wrapper function.
 *
 * 2) The old "command" interface is removed from low level drivers in the
 *    event that a queuecommand interface also exists.
 *
 * 3) Changes were automatically made to "strip" the old error handling code
 *    from all drivers that still had entrypoints referenced.  The changes
 *    consist of several different things:
 *        a) The old abort and reset entrypoints were "commented" out.
 *        b) The prototypes for the old abort and reset entrypoints were
 *           also commented out.
 *        c) The queuecommand wrapper (generated above) will always return
 *           0 in the event that the driver was not updated to use the new
 *           error handling code.
 *
 * 4) One last important point.  I realize that some driver maintainers attempt
 *    to use the same source on multiple kernel versions.  Thus the changes
 *    that this program generates are all conditional on the kernel version.
 *    Currently set to 2.4.1, but this is trivially configured in the source
 *    here.
 *
 * 5) This program essentially generates files with a .new extension that
 *    contain the edited sources.  This makes it easy as I was testing various
 *    ideas here and there - the originals were never overwritten, and I could
 *    just keep rerunning over and over again.  Trivially reconfigured if you
 *    really care to overwrite the originals.
 *
 * It is intended that these edits form a 1st draft of the changes.  This 
 * program does not attempt to update the mid-layer code to strip out the
 * locking, nor does it attempt to remove the old error handling code.
 * Furthermore, I anticipate that there are a handful of hand-edits that
 * would be required to get all of the low-level drivers up to snuff,
 * and finally, there are instances where #if/#else/#endif blocks could
 * be combined for improved readability.  Given the huge nature of these
 * tasks, having the machine generate the 1st draft is obviously a big win
 * in that there is no tedious editing required.
 */ 

/* Yes, this silly thing does run on Windows too... */
#ifdef _WIN32
#include <windows.h>
#else
#define O_BINARY 0
#include <limits.h>
#endif

#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef MAX_PATH
#define MAX_PATH PATH_MAX
#endif

/*
 * Edits to make:
 *
 *	*1) Insert stub function.
 *	*2) rename in template.
 *	*3) Prototype of stub.
 *	*4) Add defn of lock.  After include of associated header.
 *	*5) Rename io_request_lock, as required.
 *	*6) Add processing for remaining functions to be intercepted.
 *	*7) Change "command" interface to "NULL" in template in the event that a
 *	   queuecommand also exists.
 *	*8) Comment out prototype for command interface if queuecommand also
 *	   exists.
 *	*9) Eliminate function body for command interface if queuecommand also
 *	   exists.
 *	10) Duplicate shim bodies in the event of functions being used more
 *	    that once for same purpose.
 *	*11) parameters in stub fcts are not right.
 *	*12) Buslogic.c,h not handled.
 *	*13) NCR5380.c not handled - included by many drivers.  Probably needs
 *	    hand edits.
 *	*14) NCR53C9x.c not handled.
 *	15) sym53c8xx_comm.h not handled.  Lock referenced here is redundant,
 *	    needs hand edits.
 *
 * Dead sources (no longer used): mac_NCR5380.c
 *
 * Possible future stuff.  Also do conditionally dep on version:
 *	1) Remove old error handling entrypoints from templates.
 *	*2) Remove prototypes for old error handling entrypoints.
 *	*3) Comment out sources for old error handling functions.
 *	*4) Fix wrapper for non-converted drivers to return 0 indicating
 *	   success.
 *	5) Force "use_new_eh_code" flag to 1.
 */
/*
 * Fields to add:
 *
 *	last modification date.
 *	orphaned.
 */

#define NCR5380	1
#define NCR53C9X 2
#define NCR53C9X_D 4

#define NODELETE 0
#define DELETEIT 1
#define COMMENTIT 2

#define BODY1 "\n#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,1)\n/*\n * No longer used with new error handling code\n */\n"
struct hstruct
{
	char * hs_name;
	char * hs_header;
	char * hs_symbol;
	int flags;
	int fixed; /* Bitmap of things modified - not yet used. */
	int fix_exp; /* Bitmap of changed we expect - not yet used. */
	char * email;
	char * lockname;
	/*
    * The rest of these are fields from the driver template.
    */
	char *next;
	char *module;
	char *proc_dir;
	char *proc_info;
	char *name;
	char *detect;
	char *revoke;
	char *release;
	char *info;
	char *ioctl;
	char *command;
	char *queuecommand;
	char *eh_strategy_handler;
	char *eh_abort_handler;
	char *eh_device_reset_handler;
	char *eh_bus_reset_handler;
	char *eh_host_reset_handler;
	char *abort;
	char *reset;
	char *slave_attach;
	char *bios_param;
	char *select_queue_depths;
	char *can_queue;
	char *this_id;
	char *sg_tablesize;
	char *cmd_per_lun;
	char *present;
	char *unchecked_isa_dma;
	char *use_clustering;
	char *use_new_eh_code;
	char *emulated;
	char *proc_name;
};

struct hstruct drivers[] =
{
	{
		"drivers/scsi/3w-xxxx.c",
		"drivers/scsi/3w-xxxx.h",
		"TWXXXX",
		0, 0, 0,
		"",		/* Email: */
		"3w_xxxx_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"tw_scsi_proc_info",		/* host:proc_info */
		"\"3ware Storage Controller\"",		/* host:name */
		"tw_scsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"tw_scsi_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"tw_scsi_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"tw_scsi_eh_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"tw_scsi_eh_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"tw_scsi_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"256",		/* host:can_queue */
		"-1",		/* host:this_id */
		"62",		/* host:sg_tablesize */
		"(256 -2)/ 16",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"TRUE",		/* host:emulated */
		"\"3w-xxxx\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/BusLogic.c",
		"drivers/scsi/BusLogic.h",
		"BUSLOGIC",
		0, 0, 0,
		"",		/* Email: */
		"Buslogic_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"BusLogic_ProcDirectoryInfo",		/* host:proc_info */
		"\"BusLogic\"",		/* host:name */
		"BusLogic_DetectHostAdapter",		/* host:detect */
		"NULL",		/* host:revoke */
		"BusLogic_ReleaseHostAdapter",		/* host:release */
		"BusLogic_DriverInfo",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"BusLogic_QueueCommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"BusLogic_AbortCommand",		/* host:abort */
		"BusLogic_ResetCommand",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"BusLogic_BIOSDiskParameters",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"BusLogic\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/53c7,8xx.c",
		"drivers/scsi/53c7,8xx.h",
		"NCR53c7xx",
		0, 0, 0,
		"",		/* Email: */
		"53c7_8xx_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"NCR53c{7,8}xx (rel 17)\"",		/* host:name */
		"NCR53c7xx_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"NCR53c7xx_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"NCR53c7xx_abort",		/* host:abort */
		"NCR53c7xx_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"24",		/* host:can_queue */
		"7",		/* host:this_id */
		"127",		/* host:sg_tablesize */
		"3",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/53c7xx.c",
		"drivers/scsi/53c7xx.h",
		"NCR53c7xx",
		0, 0, 0,
		"",		/* Email: */
		"53c7xx_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"NCR53c{7,8}xx (rel 17)\"",		/* host:name */
		"NCR53c7xx_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"NCR53c7xx_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"NCR53c7xx_abort",		/* host:abort */
		"NCR53c7xx_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"24",		/* host:can_queue */
		"7",		/* host:this_id */
		"127",		/* host:sg_tablesize */
		"3",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/AM53C974.c",
		"drivers/scsi/AM53C974.h",
		"AM53C974",
		0, 0, 0,
		"",		/* Email: */
		"AM53C974_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"AM53C974\"",		/* host:name */
		"AM53C974_pci_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"AM53C974_release",		/* host:release */
		"AM53C974_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"AM53C974_command",		/* host:command */
		"AM53C974_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"AM53C974_abort",		/* host:abort */
		"AM53C974_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"12",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"am53c974\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/NCR53c406a.c",
		"drivers/scsi/NCR53c406a.h",
		"NCR53c406a",
		0, 0, 0,
		"",		/* Email: */
		"NCR53c406a_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"NCR53c406a\"",		/* host:name */
		"NCR53c406a_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NCR53c406a_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NCR53c406a_command",		/* host:command */
		"NCR53c406a_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"NCR53c406a_abort",		/* host:abort */
		"NCR53c406a_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NCR53c406a_biosparm",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"7",		/* host:this_id */
		"32",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"NCR53c406a\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/a2091.c",
		"drivers/scsi/a2091.h",
		"A2091_SCSI",
		0, 0, 0,
		"",		/* Email: */
		"a2091_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Commodore A2091/A590 SCSI\"",		/* host:name */
		"a2091_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"a2091_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"wd33c93_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"wd33c93_abort",		/* host:abort */
		"wd33c93_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"A2901\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/a3000.c",
		"drivers/scsi/a3000.h",
		"_A3000_SCSI",
		0, 0, 0,
		"",		/* Email: */
		"a3000_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Amiga 3000 built-in SCSI\"",		/* host:name */
		"a3000_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"a3000_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"wd33c93_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"wd33c93_abort",		/* host:abort */
		"wd33c93_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"A3000\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/advansys.c",
		"drivers/scsi/advansys.h",
		"ADVANSYS",
		0, 0, 0,
		"",		/* Email: */
		"advansys_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"advansys_proc_info",		/* host:proc_info */
		"\"advansys\"",		/* host:name */
		"advansys_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"advansys_release",		/* host:release */
		"advansys_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"advansys_command",		/* host:command */
		"advansys_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"advansys_abort",		/* host:abort */
		"advansys_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"advansys_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"advansys\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/aha152x.c",
		"drivers/scsi/aha152x.h",
		"AHA152X",
		0, 0, 0,
		"",		/* Email: */
		"aha152x_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"aha152x_proc_info",		/* host:proc_info */
		"\"Adaptec 152x SCSI driver; $Revision:2.3 $\"",		/* host:name */
		"aha152x_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"aha152x_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"aha152x_command",		/* host:command */
		"aha152x_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"aha152x_abort",		/* host:eh_abort_handler */
		"aha152x_device_reset",		/* host:eh_device_reset_handler */
		"aha152x_bus_reset",		/* host:eh_bus_reset_handler */
		"aha152x_host_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"0",		/* host:slave_attach */
		"aha152x_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"aha152x\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/aha1542.c",
		"drivers/scsi/aha1542.h",
		"AHA1542",
		0, 0, 0,
		"",		/* Email: */
		"aha1542_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Adaptec 1542\"",		/* host:name */
		"aha1542_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"aha1542_command",		/* host:command */
		"aha1542_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"aha1542_abort",		/* host:eh_abort_handler */
		"aha1542_dev_reset",		/* host:eh_device_reset_handler */
		"aha1542_bus_reset",		/* host:eh_bus_reset_handler */
		"aha1542_host_reset",		/* host:eh_host_reset_handler */
		"aha1542_old_abort",		/* host:abort */
		"aha1542_old_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"aha1542_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"8",		/* host:can_queue */
		"7",		/* host:this_id */
		"16",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"aha1542\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/aha1740.c",
		"drivers/scsi/aha1740.h",
		"AHA1740",
		0, 0, 0,
		"",		/* Email: */
		"aha1740_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"aha1740_proc_info",		/* host:proc_info */
		"\"Adaptec 174x (EISA)\"",		/* host:name */
		"aha1740_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"aha1740_command",		/* host:command */
		"aha1740_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"aha1740_abort",		/* host:abort */
		"aha1740_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"aha1740_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"32",		/* host:can_queue */
		"7",		/* host:this_id */
		"16",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"aha1740\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/aic7xxx.c",
		"drivers/scsi/aic7xxx.h",
		"AIC7XXX",
		0, 0, 0,
		"",		/* Email: */
		"aic7xxx_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"aic7xxx_proc_info",		/* host:proc_info */
		"NULL",		/* host:name */
		"aic7xxx_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"aic7xxx_release",		/* host:release */
		"aic7xxx_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"aic7xxx_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"aic7xxx_abort",		/* host:abort */
		"aic7xxx_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"aic7xxx_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"255",		/* host:can_queue */
		"-1",		/* host:this_id */
		"0",		/* host:sg_tablesize */
		"3",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/atari_scsi.c",
		"drivers/scsi/atari_scsi.h",
		"ATARI_SCSI",
		NCR5380, 0, 0,
		"",		/* Email: */
		"atari_scsi_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"atari_scsi_proc_info",		/* host:proc_info */
		"\"Atari native SCSI\"",		/* host:name */
		"atari_scsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"atari_scsi_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"atari_scsi_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"atari_scsi_abort",		/* host:abort */
		"atari_scsi_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"0",		/* host:can_queue */
		"0",		/* host:this_id */
		"0",		/* host:sg_tablesize */
		"0",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/atp870u.c",
		"drivers/scsi/atp870u.h",
		"ATP870U",
		0, 0, 0,
		"",		/* Email: */
		"atp870u_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"atp870u_proc_info",		/* host:proc_info */
		"NULL",		/* host:name */
		"atp870u_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"atp870u_release",		/* host:release */
		"atp870u_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"atp870u_command",		/* host:command */
		"atp870u_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"atp870u_abort",		/* host:abort */
		"atp870u_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"atp870u_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"32",		/* host:can_queue */
		"7",		/* host:this_id */
		"128",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/blz1230.c",
		"drivers/scsi/blz1230.h",
		"SCSI_BLZ1230",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"esp_proc_info",		/* host:proc_info */
		"\"Blizzard1230 SCSI IV\"",		/* host:name */
		"blz1230_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"blz1230_esp_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"esp_command",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp-blz1230\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/blz2060.c",
		"drivers/scsi/blz2060.h",
		"SCSI_BLZ2060",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"esp_proc_info",		/* host:proc_info */
		"\"Blizzard2060 SCSI\"",		/* host:name */
		"blz2060_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"blz2060_esp_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp-blz2060\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/cpqfcTSinit.c",
		"drivers/scsi/cpqfcTS.h",
		"CPQFCTS",
		0, 0, 0,
		"",		/* Email: */
		"cpqfcTSinit_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"cpqfcTS_proc_info",		/* host:proc_info */
		"NULL",		/* host:name */
		"cpqfcTS_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"cpqfcTS_release",		/* host:release */
		"cpqfcTS_info",		/* host:info */
		"cpqfcTS_ioctl",		/* host:ioctl */
		"NULL",		/* host:command */
		"cpqfcTS_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"cpqfcTS_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"cpqfcTS_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"cpqfcTS_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"(512 /2)",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"15",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/cyberstorm.c",
		"drivers/scsi/cyberstorm.h",
		"SCSI_CYBERSTORM",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"esp_proc_info",		/* host:proc_info */
		"\"CyberStorm SCSI\"",		/* host:name */
		"cyber_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"cyber_esp_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp-cyberstorm\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/cyberstormII.c",
		"drivers/scsi/cyberstormII.h",
		"SCSI_CYBERSTORMII",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"esp_proc_info",		/* host:proc_info */
		"\"CyberStorm Mk II SCSI\"",		/* host:name */
		"cyberII_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"cyberII_esp_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp-cyberstormII\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/dmx3191d.c",
		"drivers/scsi/dmx3191d.h",
		"DMX3191D",
		NCR5380, 0, 0,
		"",		/* Email: */
		"dmx3191d_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"dmx3191d_proc_info",		/* host:proc_info */
		"\"Domex DMX3191D\"",		/* host:name */
		"dmx3191d_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"dmx3191d_release_resources",		/* host:release */
		"dmx3191d_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"dmx3191d_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"dmx3191d_abort",		/* host:abort */
		"dmx3191d_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"32",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/dtc.c",
		"drivers/scsi/dtc.h",
		"DTC3x80",
		NCR5380, 0, 0,
		"",		/* Email: */
		"dtc_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"DTC 3180/3280 \"",		/* host:name */
		"dtc_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"dtc_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"dtc_abort",		/* host:abort */
		"dtc_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"dtc_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"32",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/eata.c",
		"drivers/scsi/eata.h",
		"EATA",
		0, 0, 0,
		"",		/* Email: */
		"eata_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"EATA/DMA 2.0x rev. \" \"6.02.00\"  \" \"",		/* host:name */
		"eata2x_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"eata2x_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"eata2x_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"eata2x_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"eata2x_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"eata2x_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"7",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/eata_dma.c",
		"drivers/scsi/eata_dma.h",
		"EATA_DMA",
		0, 0, 0,
		"",		/* Email: */
		"eata_dma_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"eata_proc_info",		/* host:proc_info */
		"\"EATA (Extended Attachment) HBA driver\"",		/* host:name */
		"eata_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"eata_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"eata_abort",		/* host:abort */
		"eata_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/eata_pio.c",
		"drivers/scsi/eata_pio.h",
		"EATA_PIO",
		0, 0, 0,
		"",		/* Email: */
		"eata_pio_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"eata_pio_proc_info",		/* host:proc_info */
		"\"EATA (Extended Attachment) PIO driver\"",		/* host:name */
		"eata_pio_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"eata_pio_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"eata_pio_abort",		/* host:abort */
		"eata_pio_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"TRUE",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/esp.c",
		"drivers/scsi/esp.h",
		"SCSI_SPARC_ESP",
		0, 0, 0,
		"",		/* Email: */
		"esp_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"&esp_proc_info",		/* host:proc_info */
		"\"Sun ESP 100/100a/200\"",		/* host:name */
		"esp_detect",		/* host:detect */
		"esp_revoke",		/* host:revoke */
		"NULL",		/* host:release */
		"esp_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"esp_command",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/fastlane.c",
		"drivers/scsi/fastlane.h",
		"SCSI_FASTLANE",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"esp_proc_info",		/* host:proc_info */
		"\"Fastlane SCSI\"",		/* host:name */
		"fastlane_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"fastlane_esp_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp-fastlane\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/fcal.c",
		"drivers/scsi/fcal.h",
		"FCAL",
		0, 0, 0,
		"",		/* Email: */
		"fcal_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"fcal_proc_info",		/* host:proc_info */
		"\"Fibre Channel Arbitrated Loop\"",		/* host:name */
		"fcal_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"fcal_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"fcp_scsi_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"fcp_scsi_abort",		/* host:eh_abort_handler */
		"fcp_scsi_dev_reset",		/* host:eh_device_reset_handler */
		"fcp_scsi_bus_reset",		/* host:eh_bus_reset_handler */
		"fcp_scsi_host_reset",		/* host:eh_host_reset_handler */
		"fcp_old_abort",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"512",		/* host:can_queue */
		"-1",		/* host:this_id */
		"1",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FCP_SCSI_USE_NEW_EH_CODE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/fd_mcs.c",
		"drivers/scsi/fd_mcs.h",
		"FD_MCS",
		0, 0, 0,
		"",		/* Email: */
		"fd_mcs_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"fd_mcs_proc_info",		/* host:proc_info */
		"NULL",		/* host:name */
		"fd_mcs_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"fd_mcs_release",		/* host:release */
		"fd_mcs_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"fd_mcs_command",		/* host:command */
		"fd_mcs_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"fd_mcs_abort",		/* host:abort */
		"fd_mcs_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"fd_mcs_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"7",		/* host:this_id */
		"64",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"fd_mcs\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/fdomain.c",
		"drivers/scsi/fdomain.h",
		"FDOMAIN_16X0",
		0, 0, 0,
		"",		/* Email: */
		"fdomain_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"fdomain_16x0_proc_info",		/* host:proc_info */
		"NULL",		/* host:name */
		"fdomain_16x0_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"fdomain_16x0_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"fdomain_16x0_command",		/* host:command */
		"fdomain_16x0_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"fdomain_16x0_abort",		/* host:abort */
		"fdomain_16x0_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"fdomain_16x0_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"6",		/* host:this_id */
		"64",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/g_NCR5380.c",
		"drivers/scsi/g_NCR5380.h",
		"GENERIC_NCR5380",
		NCR5380, 0, 0,
		"",		/* Email: */
		"g_NCR5380_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"generic_NCR5380_proc_info",		/* host:proc_info */
		"\"Generic NCR5380/NCR53C400 Scsi Driver\"",		/* host:name */
		"generic_NCR5380_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"generic_NCR5380_release_resources",		/* host:release */
		"(void *)generic_NCR5380_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"generic_NCR5380_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"generic_NCR5380_abort",		/* host:abort */
		"generic_NCR5380_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"0",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/gdth.c",
		"drivers/scsi/gdth.h",
		"GDTH",
		0, 0, 0,
		"",		/* Email: */
		"gdth_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"gdth_proc_info",		/* host:proc_info */
		"\"GDT SCSI Disk Array Controller\"",		/* host:name */
		"gdth_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"gdth_release",		/* host:release */
		"gdth_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"gdth_command",		/* host:command */
		"gdth_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"gdth_eh_abort",		/* host:eh_abort_handler */
		"gdth_eh_device_reset",		/* host:eh_device_reset_handler */
		"gdth_eh_bus_reset",		/* host:eh_bus_reset_handler */
		"gdth_eh_host_reset",		/* host:eh_host_reset_handler */
		"gdth_abort",		/* host:abort */
		"gdth_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"gdth_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"124",		/* host:can_queue */
		"-1",		/* host:this_id */
		"32",		/* host:sg_tablesize */
		"16",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"gdth\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/gvp11.c",
		"drivers/scsi/gvp11.h",
		"GVP11_SCSI",
		0, 0, 0,
		"",		/* Email: */
		"gvp11_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"GVP Series II SCSI\"",		/* host:name */
		"gvp11_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"gvp11_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"wd33c93_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"wd33c93_abort",		/* host:abort */
		"wd33c93_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"GVP11\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/ibmmca.c",
		"drivers/scsi/ibmmca.h",
		"IBMMCA",
		0, 0, 0,
		"",		/* Email: */
		"ibmmca_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"ibmmca_proc_info",		/* host:proc_info */
		"\"IBM SCSI-Subsystem\"",		/* host:name */
		"ibmmca_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"ibmmca_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"ibmmca_command",		/* host:command */
		"ibmmca_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"ibmmca_abort",		/* host:abort */
		"ibmmca_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"ibmmca_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"16",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"ibmmca\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/ide-scsi.c",
		"drivers/scsi/ide-scsi.h",
		"IDESCSI",
		0, 0, 0,
		"",		/* Email: */
		"ide_scsi_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"idescsi\"",		/* host:name */
		"idescsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"idescsi_release",		/* host:release */
		"idescsi_info",		/* host:info */
		"idescsi_ioctl",		/* host:ioctl */
		"NULL",		/* host:command */
		"idescsi_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"idescsi_abort",		/* host:abort */
		"idescsi_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"idescsi_bios",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"10",		/* host:can_queue */
		"-1",		/* host:this_id */
		"256",		/* host:sg_tablesize */
		"5",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"TRUE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/imm.c",
		"drivers/scsi/imm.h",
		"IMM",
		0, 0, 0,
		"",		/* Email: */
		"imm_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"imm_proc_info",		/* host:proc_info */
		"\"Iomega VPI2 (imm) interface\"",		/* host:name */
		"imm_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"0",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"imm_command",		/* host:command */
		"imm_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"imm_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"imm_reset",		/* host:eh_bus_reset_handler */
		"imm_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"imm_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"imm\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/in2000.c",
		"drivers/scsi/in2000.h",
		"IN2000",
		0, 0, 0,
		"",		/* Email: */
		"in2000_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"in2000_proc_info",		/* host:proc_info */
		"\"Always IN2000\"",		/* host:name */
		"in2000_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"in2000_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"in2000_abort",		/* host:abort */
		"in2000_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"in2000_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"in2000\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/ini9100u.c",
		"drivers/scsi/ini9100u.h",
		"INI9100U",
		0, 0, 0,
		"",		/* Email: */
		"ini9100u_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Initio INI-9X00U/UW SCSI device driver; Revision:1.03g\"",		/* host:name */
		"i91u_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"i91u_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"i91u_command",		/* host:command */
		"i91u_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"i91u_abort",		/* host:abort */
		"i91u_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"i91u_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"INI9100U\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/inia100.c",
		"drivers/scsi/inia100.h",
		"INIA100",
		0, 0, 0,
		"",		/* Email: */
		"inia100_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Initio INI-A100U2W SCSI device driver; Revision:1.02c\"",		/* host:name */
		"inia100_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"inia100_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"inia100_command",		/* host:command */
		"inia100_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"inia100_abort",		/* host:abort */
		"inia100_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"inia100_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"INIA100\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/ips.c",
		"drivers/scsi/ips.h",
		"IPS",
		0, 0, 0,
		"",		/* Email: */
		"ips_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"ips_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"ips_release",		/* host:release */
		"ips_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"ips_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"ips_eh_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"ips_eh_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"ips_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"0",		/* host:can_queue */
		"-1",		/* host:this_id */
		"17",		/* host:sg_tablesize */
		"16",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/mac53c94.c",
		"drivers/scsi/mac53c94.h",
		"SCSI_MAC53C94",
		0, 0, 0,
		"",		/* Email: */
		"mac53c94_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"53C94\"",		/* host:name */
		"mac53c94_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"mac53c94_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"mac53c94_command",		/* host:command */
		"mac53c94_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"mac53c94_abort",		/* host:abort */
		"mac53c94_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"53c94\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/mac_scsi.c",
		"drivers/scsi/mac_scsi.h",
		"MAC_NCR5380",
		NCR5380, 0, 0,
		"",		/* Email: */
		"mac_NCR5380_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Macintosh NCR5380 SCSI\"",		/* host:name */
		"macscsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"macscsi_release",		/* host:release */
		"macscsi_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"macscsi_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"macscsi_abort",		/* host:abort */
		"macscsi_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/mac_esp.c",
		"drivers/scsi/mac_esp.h",
		"SCSI_MAC_ESP",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Mac 53C9x SCSI\"",		/* host:name */
		"mac_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"esp_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/mac_scsi.c",
		"drivers/scsi/mac_scsi.h",
		"MAC_NCR5380",
		0, 0, 0,
		"",		/* Email: */
		"mac_scsi_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Macintosh NCR5380 SCSI\"",		/* host:name */
		"macscsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"macscsi_release",		/* host:release */
		"macscsi_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"macscsi_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"macscsi_abort",		/* host:abort */
		"macscsi_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/mca_53c9x.c",
		"drivers/scsi/mca_53c9x.h",
		"MCA_53C9X",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"NCR 53c9x SCSI\"",		/* host:name */
		"mca_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"mca_esp_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/megaraid.c",
		"drivers/scsi/megaraid.h",
		"MEGARAID",
		0, 0, 0,
		"",		/* Email: */
		"megaraid_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"megaraid_proc_info",		/* host:proc_info */
		"\"MegaRAID\"",		/* host:name */
		"megaraid_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"megaraid_release",		/* host:release */
		"megaraid_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"megaraid_command",		/* host:command */
		"megaraid_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"megaraid_abort",		/* host:abort */
		"megaraid_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"megaraid_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"127",		/* host:can_queue */
		"7",		/* host:this_id */
		"0x1A",		/* host:sg_tablesize */
		"63",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/mesh.c",
		"drivers/scsi/mesh.h",
		"SCSI_MESH",
		0, 0, 0,
		"",		/* Email: */
		"mesh_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"MESH\"",		/* host:name */
		"mesh_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"mesh_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"mesh_command",		/* host:command */
		"mesh_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"mesh_abort",		/* host:abort */
		"mesh_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"20",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"mesh\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/mvme147.c",
		"drivers/scsi/mvme147.h",
		"MVME147_SCSI",
		0, 0, 0,
		"",		/* Email: */
		"mvme147_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"MVME147 built-in SCSI\"",		/* host:name */
		"mvme147_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"mvme147_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"wd33c93_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"wd33c93_abort",		/* host:abort */
		"wd33c93_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"MVME147\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/ncr53c8xx.c",
		"drivers/scsi/ncr53c8xx.h",
		"NCR53C8XX",
		0, 0, 0,
		"",		/* Email: */
		"ncr53c8xx_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"\"",		/* host:name */
		"ncr53c8xx_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"ncr53c8xx_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"ncr53c8xx_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"ncr53c8xx_abort",		/* host:abort */
		"ncr53c8xx_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"(8* (8)  + 2* (16) )",		/* host:can_queue */
		"7",		/* host:this_id */
		"((127) )",		/* host:sg_tablesize */
		"((8) )",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/oktagon_esp.c",
		"drivers/scsi/oktagon_esp.h",
		"SCSI_OKTAGON_ESP",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"&esp_proc_info",		/* host:proc_info */
		"\"BSC Oktagon SCSI\"",		/* host:name */
		"oktagon_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"oktagon_esp_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp-oktagon\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/pas16.c",
		"drivers/scsi/pas16.h",
		"MV_PAS16",
		NCR5380, 0, 0,
		"",		/* Email: */
		"pas16_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Pro Audio Spectrum-16 SCSI\"",		/* host:name */
		"pas16_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"pas16_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"pas16_abort",		/* host:abort */
		"pas16_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"pas16_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"32",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/pci2000.c",
		"drivers/scsi/pci2000.h",
		"PCI2000",
		0, 0, 0,
		"",		/* Email: */
		"pci2000_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"PCI-2000 SCSI Intelligent Disk Controller\"",		/* host:name */
		"Pci2000_Detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"Pci2000_Release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"Pci2000_Command",		/* host:command */
		"Pci2000_QueueCommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"Pci2000_Abort",		/* host:abort */
		"Pci2000_Reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"Pci2000_BiosParam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"-1",		/* host:this_id */
		"16",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"pci2000\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/pci2220i.c",
		"drivers/scsi/pci2220i.h",
		"PCI2220I",
		0, 0, 0,
		"",		/* Email: */
		"pci2220i_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"PCI-2220I/PCI-2240I\"",		/* host:name */
		"Pci2220i_Detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"Pci2220i_Release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"Pci2220i_Command",		/* host:command */
		"Pci2220i_QueueCommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"Pci2220i_Abort",		/* host:abort */
		"Pci2220i_Reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"Pci2220i_BiosParam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"pci2220i\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/pluto.c",
		"drivers/scsi/pluto.h",
		"PLUTO",
		0, 0, 0,
		"",		/* Email: */
		"pluto_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Sparc Storage Array 100/200\"",		/* host:name */
		"pluto_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"pluto_release",		/* host:release */
		"pluto_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"fcp_scsi_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"fcp_scsi_abort",		/* host:eh_abort_handler */
		"fcp_scsi_dev_reset",		/* host:eh_device_reset_handler */
		"fcp_scsi_bus_reset",		/* host:eh_bus_reset_handler */
		"fcp_scsi_host_reset",		/* host:eh_host_reset_handler */
		"fcp_old_abort",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"254",		/* host:can_queue */
		"-1",		/* host:this_id */
		"1",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FCP_SCSI_USE_NEW_EH_CODE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/ppa.c",
		"drivers/scsi/ppa.h",
		"PPA",
		0, 0, 0,
		"",		/* Email: */
		"ppa_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"ppa_proc_info",		/* host:proc_info */
		"\"Iomega VPI0 (ppa) interface\"",		/* host:name */
		"ppa_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"0",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"ppa_command",		/* host:command */
		"ppa_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"ppa_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"ppa_reset",		/* host:eh_bus_reset_handler */
		"ppa_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"ppa_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"ppa\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/psi240i.c",
		"drivers/scsi/psi240i.h",
		"PSI240I",
		0, 0, 0,
		"",		/* Email: */
		"psi240i_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"PSI-240I EIDE Disk Controller\"",		/* host:name */
		"Psi240i_Detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"Psi240i_Command",		/* host:command */
		"Psi240i_QueueCommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"Psi240i_Abort",		/* host:abort */
		"Psi240i_Reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"Psi240i_BiosParam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_NONE",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"psi240i\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/qla1280.c",
		"drivers/scsi/qla1280.h",
		"QLA1280_LINUX_TEMPLATE",
		0, 0, 0,
		"",		/* Email: */
		"qla1280_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"qla1280_proc_info",		/* host:proc_info */
		"\"Qlogic ISP 128080\"",		/* host:name */
		"qla1280_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"qla1280_release",		/* host:release */
		"qla1280_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"qla1280_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"qla1280_abort",		/* host:abort */
		"qla1280_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"255",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"3",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/qlogicfas.c",
		"drivers/scsi/qlogicfas.h",
		"QLOGICFAS",
		0, 0, 0,
		"",		/* Email: */
		"qlogicfas_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"qlogicfas_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"qlogicfas_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"qlogicfas_command",		/* host:command */
		"qlogicfas_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"qlogicfas_abort",		/* host:abort */
		"qlogicfas_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"qlogicfas_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"0",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/qlogicfc.c",
		"drivers/scsi/qlogicfc.h",
		"QLOGICFC",
		0, 0, 0,
		"",		/* Email: */
		"qlogicfc_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"isp2x00_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"isp2x00_release",		/* host:release */
		"isp2x00_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"isp2x00_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"isp2x00_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"isp2x00_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"isp2x00_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"127",		/* host:can_queue */
		"-1",		/* host:this_id */
		"(3  + ((( 127  ) > 0) ? 7 *(( 127  ) - 1):0))",		/* host:sg_tablesize */
		"8",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/qlogicisp.c",
		"drivers/scsi/qlogicisp.h",
		"QLOGICISP",
		0, 0, 0,
		"",		/* Email: */
		"qlogicisp_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"isp1020_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"isp1020_release",		/* host:release */
		"isp1020_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"isp1020_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"isp1020_abort",		/* host:abort */
		"isp1020_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"isp1020_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"63",		/* host:can_queue */
		"-1",		/* host:this_id */
		"(4 + (( 63  ) > 0) ? 7*(( 63  ) - 1):0)",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/qlogicpti.c",
		"drivers/scsi/qlogicpti.h",
		"QLOGICPTI",
		0, 0, 0,
		"",		/* Email: */
		"qlogicpti_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"qlogicpti_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"qlogicpti_release",		/* host:release */
		"qlogicpti_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"qlogicpti_queuecommand_slow",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"qlogicpti_abort",		/* host:abort */
		"qlogicpti_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"255",		/* host:can_queue */
		"7",		/* host:this_id */
		"(4 + (( 255  ) > 0) ? 7*(( 255  ) - 1):0)",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/scsi_debug.c",
		"drivers/scsi/scsi_debug.h",
		"SCSI_DEBUG",
		0, 0, 0,
		"",		/* Email: */
		"scsi_debug_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"scsi_debug_proc_info",		/* host:proc_info */
		"\"SCSI DEBUG\"",		/* host:name */
		"scsi_debug_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"scsi_debug_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"scsi_debug_abort",		/* host:abort */
		"scsi_debug_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsi_debug_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"255",		/* host:can_queue */
		"7",		/* host:this_id */
		"16",		/* host:sg_tablesize */
		"3",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/seagate.c",
		"drivers/scsi/seagate.h",
		"SEAGATE_ST0X",
		0, 0, 0,
		"",		/* Email: */
		"seagate_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"seagate_st0x_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"seagate_st0x_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"seagate_st0x_command",		/* host:command */
		"seagate_st0x_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"seagate_st0x_abort",		/* host:abort */
		"seagate_st0x_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/sgiwd93.c",
		"drivers/scsi/sgiwd93.h",
		"SGIWD93_SCSI",
		0, 0, 0,
		"",		/* Email: */
		"sgiwd93_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"SGI WD93\"",		/* host:name */
		"sgiwd93_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"sgiwd93_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"wd33c93_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"wd33c93_abort",		/* host:abort */
		"wd33c93_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"8",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"SGIWD93\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/sim710.c",
		"drivers/scsi/sim710.h",
		"SIM710_SCSI",
		0, 0, 0,
		"",		/* Email: */
		"sim710_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Simple 53c710\"",		/* host:name */
		"sim710_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"sim710_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"sim710_abort",		/* host:eh_abort_handler */
		"sim710_dev_reset",		/* host:eh_device_reset_handler */
		"sim710_bus_reset",		/* host:eh_bus_reset_handler */
		"sim710_host_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"8",		/* host:can_queue */
		"7",		/* host:this_id */
		"128",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"sim710\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/sun3_scsi.c",
		"drivers/scsi/sun3_scsi.h",
		"SUN3_NCR5380",
		NCR5380, 0, 0,
		"",		/* Email: */
		"sun3_scsi_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Sun3 NCR5380 SCSI\"",		/* host:name */
		"sun3scsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"sun3scsi_release",		/* host:release */
		"sun3scsi_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"sun3scsi_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"sun3scsi_abort",		/* host:abort */
		"sun3scsi_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/sun3x_esp.c",
		"drivers/scsi/sun3x_esp.h",
		"SCSI_SUN3X_ESP",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"&esp_proc_info",		/* host:proc_info */
		"\"Sun ESP 100/100a/200\"",		/* host:name */
		"sun3x_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"esp_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"esp_command",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/sym53c416.c",
		"drivers/scsi/sym53c416.h",
		"SYM53C416",
		0, 0, 0,
		"",		/* Email: */
		"sym53c416_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Symbios Logic 53c416\"",		/* host:name */
		"sym53c416_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"sym53c416_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"sym53c416_command",		/* host:command */
		"sym53c416_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"sym53c416_abort",		/* host:abort */
		"sym53c416_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"sym53c416_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"7",		/* host:this_id */
		"32",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"sym53c416\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/sym53c8xx.c",
		"drivers/scsi/sym53c8xx.h",
		"SYM53C8XX",
		0, 0, 0,
		"",		/* Email: */
		"sym53c8xx_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"\"",		/* host:name */
		"sym53c8xx_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"sym53c8xx_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"sym53c8xx_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"sym53c8xx_abort",		/* host:abort */
		"sym53c8xx_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"scsicam_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"(8* (8)  + 2* (16) )",		/* host:can_queue */
		"7",		/* host:this_id */
		"((127) )",		/* host:sg_tablesize */
		"((8) )",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/t128.c",
		"drivers/scsi/t128.h",
		"TRANTOR_T128",
		NCR5380, 0, 0,
		"",		/* Email: */
		"t128_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"Trantor T128/T128F/T228\"",		/* host:name */
		"t128_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"t128_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"t128_abort",		/* host:abort */
		"t128_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"t128_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"32",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"2",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/tmscsim.c",
		"drivers/scsi/dc390.h",
		"DC390_T",
		0, 0, 0,
		"",		/* Email: */
		"tmscsim_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"DC390_proc_info",		/* host:proc_info */
		"\"Tekram DC390/AM53C974\"  \" V\" \"2.0f 2000-12-20\"",		/* host:name */
		"DC390_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"DC390_queue_command",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"DC390_abort",		/* host:abort */
		"DC390_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"DC390_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"42",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"16",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"tmscsim\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/u14-34f.c",
		"drivers/scsi/u14-34f.h",
		"ULTRASTOR_14_34F",
		0, 0, 0,
		"",		/* Email: */
		"u14_34f_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"UltraStor 14F/34F rev. \" \"6.02.00\"  \" \"",		/* host:name */
		"u14_34f_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"u14_34f_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"u14_34f_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"u14_34f_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"u14_34f_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"u14_34f_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"7",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/ultrastor.c",
		"drivers/scsi/ultrastor.h",
		"ULTRASTOR_14F",
		0, 0, 0,
		"",		/* Email: */
		"ultrastor_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"UltraStor 14F/24F/34F\"",		/* host:name */
		"ultrastor_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"ultrastor_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"ultrastor_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"ultrastor_abort",		/* host:abort */
		"ultrastor_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"ultrastor_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"0",		/* host:this_id */
		"16",		/* host:sg_tablesize */
		"5",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/scsi/wd7000.c",
		"drivers/scsi/wd7000.h",
		"WD7000",
		0, 0, 0,
		"",		/* Email: */
		"wd7000_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"wd7000_proc_info",		/* host:proc_info */
		"\"Western Digital WD-7000\"",		/* host:name */
		"wd7000_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"wd7000_command",		/* host:command */
		"wd7000_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"wd7000_abort",		/* host:abort */
		"wd7000_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"wd7000_biosparam",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"16",		/* host:can_queue */
		"7",		/* host:this_id */
		"16",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"wd7000\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/NCR5380.c",
		"drivers/scsi/NCR5380.h",
		"",
		0, 0, 0,
		"",		/* Email: */
		"NCR5380_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"NULL",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"NULL",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"",		/* host:proc_name */
	},
	{
		"drivers/scsi/NCR53C9x.c",
		"drivers/scsi/NCR53C9x.h",
		"",
		NCR53C9X_D, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"NULL",		/* host:name */
		"NULL",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"NULL",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"",		/* host:can_queue */
		"",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"",		/* host:cmd_per_lun */
		"",		/* host:present */
		"TRUE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"",		/* host:proc_name */
	},
	{
		"drivers/scsi/jazz_esp.c",
		"drivers/scsi/jazz_esp.h",
		"SCSI_JAZZ_ESP",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"&esp_proc_info",		/* host:proc_info */
		"\"ESP 100/100a/200\"",		/* host:name */
		"jazz_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"esp_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"esp_command",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp\"",		/* host:proc_name */
	},
	{
		"drivers/scsi/dec_esp.c",
		"drivers/scsi/dec_esp.h",
		"SCSI_DEC_ESP",
		NCR53C9X, 0, 0,
		"",		/* Email: */
		"NCR53C9x_LOCK",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"&esp_proc_info",		/* host:proc_info */
		"\"NCR53C94\"",		/* host:name */
		"dec_esp_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"NULL",		/* host:release */
		"esp_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"esp_command",		/* host:command */
		"esp_queue",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"esp_abort",		/* host:abort */
		"esp_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"7",		/* host:can_queue */
		"7",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"DISABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"esp\"",		/* host:proc_name */
	},
	{
		"drivers/i2o/i2o_scsi.c",
		"drivers/i2o/i2o_scsi.h",
		"I2OSCSI",
		0, 0, 0,
		"",		/* Email: */
		"i2o_scsi_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"I2O SCSI Layer\"",		/* host:name */
		"i2o_scsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"i2o_scsi_release",		/* host:release */
		"i2o_scsi_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"i2o_scsi_command",		/* host:command */
		"i2o_scsi_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"NULL",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"NULL",		/* host:eh_host_reset_handler */
		"i2o_scsi_abort",		/* host:abort */
		"i2o_scsi_reset",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"i2o_scsi_bios_param",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"4",		/* host:can_queue */
		"15",		/* host:this_id */
		"8",		/* host:sg_tablesize */
		"6",		/* host:cmd_per_lun */
		"",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"ENABLE_CLUSTERING",		/* host:use_clustering */
		"FALSE",		/* host:use_new_eh_code */
		"FALSE",		/* host:emulated */
		"\"i2o_scsi\"",		/* host:proc_name */
	},
	{
		"drivers/usb/microtek.c",
		"drivers/usb/microtek.h",
		"NOTUSED",
		0, 0, 0,
		"",		/* Email: */
		"microtek_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"NULL",		/* host:proc_info */
		"\"microtek\"",		/* host:name */
		"mts_scsi_detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"mts_scsi_release",		/* host:release */
		"NULL",		/* host:info */
		"NULL",		/* host:ioctl */
		"NULL",		/* host:command */
		"mts_scsi_queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"mts_scsi_abort",		/* host:eh_abort_handler */
		"NULL",		/* host:eh_device_reset_handler */
		"NULL",		/* host:eh_bus_reset_handler */
		"mts_scsi_host_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"-1",		/* host:this_id */
		"",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"FALSE",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"TRUE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
	{
		"drivers/usb/storage/scsiglue.c",
		"drivers/usb/storage/scsiglue.h",
		"NOTUSED",
		0, 0, 0,
		"",		/* Email: */
		"scsiglue_lock",		/* lock name */
		"NULL",		/* host:next */
		"NULL",		/* host:module */
		"NULL",		/* host:proc_dir */
		"proc_info",		/* host:proc_info */
		"\"usb-storage\"",		/* host:name */
		"detect",		/* host:detect */
		"NULL",		/* host:revoke */
		"release",		/* host:release */
		"host_info",		/* host:info */
		"NULL",		/* host:ioctl */
		"command",		/* host:command */
		"queuecommand",		/* host:queuecommand */
		"NULL",		/* host:eh_strategy_handler */
		"command_abort",		/* host:eh_abort_handler */
		"bus_reset",		/* host:eh_device_reset_handler */
		"bus_reset",		/* host:eh_bus_reset_handler */
		"host_reset",		/* host:eh_host_reset_handler */
		"NULL",		/* host:abort */
		"NULL",		/* host:reset */
		"NULL",		/* host:slave_attach */
		"NULL",		/* host:bios_param */
		"NULL",		/* host:select_queue_depths */
		"1",		/* host:can_queue */
		"-1",		/* host:this_id */
		"SG_ALL",		/* host:sg_tablesize */
		"1",		/* host:cmd_per_lun */
		"0",		/* host:present */
		"FALSE",		/* host:unchecked_isa_dma */
		"TRUE",		/* host:use_clustering */
		"TRUE",		/* host:use_new_eh_code */
		"TRUE",		/* host:emulated */
		"NULL",		/* host:proc_name */
	},
};

int delete(char * from, char * to)
{
	char * pnt2;
	char * pnt1;
	for(pnt1 = from, pnt2 = to; *pnt2; )
	{
		*pnt1++ = *pnt2++;
	}
	*pnt1 = '\0';
	return 0;
}

int inject(char * position, char * contents)
{
	char * pnt2;
	int len = strlen(contents);
	for(pnt2 = position + strlen(position) + strlen(contents);
	    pnt2 - len >= position;
	    pnt2--)
	{
		*pnt2 = pnt2[-len];
	}
	memcpy(position, contents, len);
	return 0;
}

int clean_template(char * start, char * end, char * tmpl_name, int del_type)
{
	char * pnt;
	char * pnt2;

	/*
	 * Remove an entry from a template initializer.
	 * For new style initializers, remove the entire
	 * line.  For old style initializers, change to
	 * a NULL.
	 */
	pnt = start - 1;
	/*
	 * Walk backwards, skipping whitespace.  See if a ':' is the
	 * 1st thing we see.
	 */
	while( *pnt == ' ' || *pnt == '\t' || *pnt == '\r'
	       || *pnt == '\n' || *pnt == '\\' )
	{
		*pnt--;
	}

	/*
	 * Make sure this is what we really expect.
	 */
	if( *pnt != ':' )
	{
		goto old_style;
	}

	/*
	 * Backspace over : and whitespace.
	 */
	pnt--;

	while( *pnt == ' ' || *pnt == '\t' || *pnt == '\r'
	       || *pnt == '\n' || *pnt == '\\' )
	{
		*pnt--;
	}

	if( strncmp(pnt - strlen(tmpl_name) + 1, tmpl_name, strlen(tmpl_name)) != 0 )
	{
		goto old_style;
	}

	if( del_type == COMMENTIT )
	{
		char symbol[64];
		char * pnt3;
		char * pnt4;

		pnt4 = symbol;
		*pnt4++ = '_';
		*pnt4++ = '_';
		for(pnt3 = start; pnt3 < end; pnt3++)
		{
			*pnt4++ = toupper(*pnt3);
		}
		*pnt4++ = '\0';

		/*
		 * skip backwards over the full field name.
		 */
		pnt = pnt - strlen(tmpl_name) + 1;
		/*
		 * OK, now find true ending point.  Skip forward from end until
		 * we find a ',' or a '}'.
		 */
		pnt2 = end;
		if( *pnt2 == ',' )
		{
			*pnt2++;
		}

		delete(pnt, pnt2);
		inject(pnt, symbol);
	}
	else
	{
		/*
		 * New style template.  Backspace until we see a ',' or a '{'.
		 */
		while( *pnt != ',' && *pnt != '{' )
		{
			*pnt--;
		}
		
		/*
		 * OK, now find true ending point.  Skip forward from end until
		 * we find a ',' or a '}'.
		 */
		pnt2 = end;
		while( *pnt2 != ',' && *pnt2 != '}' )
		{
			*pnt2++;
		}
		delete(pnt, pnt2);
	}

	return 0;

old_style:
	/*
	 * This is an old command interface, and
	 * we have a queuecommand.  Change to
	 * a NULL pointer.
	 */
	delete(start, end);
	inject(start, "NULL");
	return 0;
}

int insert_new_proto(char * start, char * end)
{
	char * pnt;
	char * pnt2;
	char * eol;
	char   newproto[2048];

	memset(newproto, 0, sizeof(newproto));

	/*
	 * Skip forward until we hit a newline, that becomes the insert
	 * point.
	 */
	for(pnt2 = end; *pnt2 != '\n'; pnt2++)
		continue;
	pnt2++;
	
	/*
	 * Now back up over things like "extern", "static", etc, until
	 * we hit something else.
	 */
	eol = NULL;
	for(pnt = start; *pnt; pnt--)
	{
		if( *pnt == ';' || *pnt == ':' || *pnt == '"' )
		{
			break;
		}
		if( *pnt == '\n') eol = pnt;
	}

	/*
	 * If we can, advance forward to the next start of a line.
	 */
	if( eol != NULL )
	{
		pnt = eol;
	}

	memcpy(newproto, pnt, start - pnt);
	strcat(newproto, "__");
	memcpy(newproto + (start - pnt) + 2, start, pnt2 - start);

	inject(pnt2, newproto);
	return 0;
}

int delete_proto(char * start, char * end, const char * funcname, 
		 char * fctname, int deltype)
{
	char * pnt;
	char * pnt2;
	char * eol;

	/*
	 * Skip forward until we hit a newline, that becomes the insert
	 * point.
	 */
	for(pnt2 = end; *pnt2 != '\n'; pnt2++)
		continue;
	pnt2++;
	
	/*
	 * Now back up over things like "extern", "static", etc, until
	 * we hit something else.
	 */
	eol = NULL;
	for(pnt = start; *pnt; pnt--)
	{
		if( *pnt == ';' || *pnt == ':' || *pnt == '"' || *pnt == '#' )
		{
			break;
		}
		if( *pnt == '\n') eol = pnt;
	}

	/*
	 * If we can, advance forward to the next start of a line.
	 */
	if( eol != NULL )
	{
		pnt = eol;
	}

	if( deltype == COMMENTIT )
	{
		char insert_str[1024];
		char define_name[32];
		char * pnt3;
		const char * pnt4;

		pnt3 = define_name;
		*pnt3++ = '_';
		*pnt3++ = '_';
		for(pnt4 = funcname; *pnt4; pnt4++)
		{
			*pnt3++ = toupper(*pnt4);
		}

		
		sprintf(insert_str, "#define %s %s:%s,\n#else\n#define %s\n#endif\n", define_name, fctname, funcname, define_name);
		inject(pnt2, insert_str);
		inject(pnt, "\n\n#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,1)");
		return strlen(insert_str) + strlen("\n#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,1)\n");
	}
	else
	{
		delete(pnt, pnt2);
		inject(pnt, "\n");
	}
	return 0;
}

int insert_new_body(char * start, char * end, const char * funcname,
		    const char * lockname,
		    const char * proto_parms,
		    const char * arglist,
		    int rtn_valid)
{
	char newbody[2048];
	char * eol;
	char * pnt;

	memset(newbody, 0, sizeof(newbody));

	/*
	 * Same as with a protototype, to a degree.  We copy everything from
	 * the function header, and insert a new body.
	 */
	/*
	 * Now back up over things like "extern", "static", etc, until
	 * we hit something else.
	 */
	eol = NULL;
	for(pnt = start; *pnt; pnt--)
	{
		if( *pnt == ';' || *pnt == '}' || *pnt == ':' || *pnt == '"' )
		{
			break;
		}
		if( pnt[0] == '/' && pnt[-1] == '*' )
		{
			break;
		}
		if( *pnt == '\n') eol = pnt;
	}

	/*
	 * If we can, advance forward to the next start of a line.
	 */
	if( eol != NULL )
	{
		pnt = eol;
	}

	memcpy(newbody, pnt, start - pnt);
	strcat(newbody, "__");
	strcat(newbody, funcname);
	strcat(newbody, "(");
	strcat(newbody, proto_parms);
	strcat(newbody, ")\n");
	strcat(newbody, "{\n");
	if( rtn_valid )
	{
		strcat(newbody, "\tint rtn;\n");
	}
	else
	{
		strcat(newbody, "\tint rtn = 0;\n");
		strcat(newbody, "\t/* Note: ");
		strcat(newbody, funcname);
		strcat(newbody, " function not guaranteed to have valid return value */\n");
	}
	strcat(newbody,"#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,1)\n\t");
	if( rtn_valid )
	{
		strcat(newbody,"rtn = ");
	}
	strcat(newbody,funcname);
	strcat(newbody,"(");
	strcat(newbody,arglist);
	strcat(newbody,");\n#else\n");
	strcat(newbody,"\tunsigned long flags;\n\tspin_lock_irqsave(&");
	strcat(newbody,lockname);
	strcat(newbody,", flags);\n\t");
	if( rtn_valid )
	{
		strcat(newbody,"rtn = ");
	}
	strcat(newbody,funcname);
	strcat(newbody,"(");
	strcat(newbody,arglist);
	strcat(newbody,");\n\tspin_lock_irqrestore(&");
	strcat(newbody,lockname);
	strcat(newbody,", flags);\n");
	strcat(newbody,"#endif\n");
	strcat(newbody,"\treturn rtn;\n");
	strcat(newbody, "}\n");

	inject(pnt, newbody);

	return strlen(newbody);
}

int delete_body(char * start, char * end, int deltype)
{
	char newbody[2048];
	char * eol;
	char * pnt;
	char * pnt2;
	int depth;

	/*
	 * Same as with a protototype, to a degree.  We copy everything from
	 * the function header, and insert a new body.
	 */
	/*
	 * Now back up over things like "extern", "static", etc, until
	 * we hit something else.
	 */
	eol = NULL;
	for(pnt = start; *pnt; pnt--)
	{
		if( *pnt == ';' || *pnt == '}' || *pnt == ':' || *pnt == '"' 
			|| *pnt == '#' )
		{
			break;
		}
		if( *pnt == '\n') eol = pnt;
	}

	/*
	 * If we can, advance forward to the next start of a line.
	 */
	if( eol != NULL )
	{
		pnt = eol;
	}

	depth = 0;
	for(pnt2 = end; *pnt2; pnt2++)
	{
		if( *pnt2 == '{' ) depth++;
		if( *pnt2 == '}' ) 
		{
			depth--;
			if( depth == 0 )
				break;
		}
	}

	pnt2++;
	if( deltype == COMMENTIT )
	{
		/*
		 * Tweak the positions a little bit.
		 */
		*pnt2++;
		while(*pnt == '\n') pnt++;

		inject(pnt2, "#endif\n");
		inject(pnt, BODY1);
		return strlen("#endif\n") + strlen(BODY1);
	}
	else
	{
		delete(pnt, pnt2);
	}

	return 0;
}

int process_shim(struct hstruct * hpnt, char * buffer, const char * funcname,
		 int del,
		 const char * proto_parms,
		 const char * arglist,
		 int rtn_valid,
		 char * fctname)
{
	char * pnt;
	char * pnt2;
	int result;
	int nesting;

	if( strcmp(funcname, "NULL") == 0 )
	{
		return 0;
	}

	/*
	 * Start by searching the buffer for funcname.
	 */
	for(pnt=buffer; *pnt; pnt++)
	{
		/*
		 * For performance.
		 */
		if(*pnt != funcname[0] )
		{
			continue;
		}
		if( strncmp(pnt, funcname, strlen(funcname)) != 0 )
		{
			continue;
		}

		/*
		 * Found something interesting.  Now figure out what it is.
		 * It could be one of several things:
		 *
		 * 1) It could be a prototype.
		 * 2) It could be the function definition.
		 * 3) It could be an unrelated identifier.
		 * 4) It could be part of a host template initializer.
		 * 5) It could be embedded in a comment.
		 * 6) It could be embedded in a diagnostic message for
		 * display.
		 *
		 * We will use a number of heuristics to narrow it down
		 * to one possibility.  We only care about 1, 2, and 4.
		 * The rest are irrelevant.
		 */

		/*
		 * Start by ruling out 3.  This is generally quite easy.
		 */
		pnt2 = pnt - 1;
		if( *pnt2 >= 'a' && *pnt2 <= 'z' ) continue;
		if( *pnt2 >= 'A' && *pnt2 <= 'Z' ) continue;
		if( *pnt2 >= '0' && *pnt2 <= '9' ) continue;
		if( *pnt2 == '_' ) continue;

		pnt2 = pnt + strlen(funcname);
		if( *pnt2 >= 'a' && *pnt2 <= 'z' ) continue;
		if( *pnt2 >= 'A' && *pnt2 <= 'Z' ) continue;
		if( *pnt2 >= '0' && *pnt2 <= '9' ) continue;
		if( *pnt2 == '_' ) continue;

		/*
		 * OK, it is something relevant.
		 */
		fprintf(stdout, "\tFound %s at %d\n", funcname, pnt - buffer);

		/*
		 * Next, see if it is a template initializer.  We would
		 * expect to *NOT* see a () as the next significant character.
		 * Scan forward and see what we see...
		 */
		result = FALSE;
		while(1==1)
		{
			if( *pnt2 == '(' || *pnt2 == '"' || *pnt2 == ':' ) break; /* Nope. */
			if( *pnt2 == '\\' && (pnt2[1] == 'n' )) break;
			if( *pnt2 == '*' && (pnt2[1] == '/' )) break;
			if( *pnt2 == ' ' || *pnt2 == '\t' || *pnt2 == '\r'
			    || *pnt2 == '\n' || *pnt2 == '\\' )
			{
				*pnt2++;
				continue;
			}
			if( pnt2[0] == '/' && pnt2[1] == '*' )
			{
				/*
				 * Comment - skip over.
				 */
				while(pnt2[0] != '*' || pnt2[1] != '/' )

				{
					pnt2++;
				}
				pnt2 += 2;
				continue;
			}

			if( *pnt2 == ',' )
			{
				/*
				 * Most likely a template initializer.
				 * Inject a "__" at the pointer.
				 */
				result = TRUE;
				break;
			}
			/*
			 * Beats me.  Assume that it is not what we are looking
			 * for and bail.
			 */
			break;
		}
		if( result == TRUE )
		{
			if( del != 0 )
			{
				clean_template(pnt, pnt + strlen(funcname),
					       fctname, del);
				continue;
			}
			/*
			 * This is definitely a template initializer.
			 * Shift the buffer up by two bytes from pnt onwards,
			 * and inject two "_" characters.
			 */
			inject(pnt, "__");
			/*
			 * Skip past this puppy.
			 */
			pnt += 3;
			continue;
		}

		/*
		 * Ruled that out.  All that should be left is either
		 * a prototype or a actual function definition.  This should
		 * be easy to ascertain - next significant character should
		 * be a '('.
		 */
		pnt2 = pnt + strlen(funcname);
		result = FALSE;
		while(1==1)
		{
			if( *pnt2 == '"' || *pnt2 == ':' ) break; /* Nope. */
			if( *pnt2 == '\\' && (pnt2[1] == 'n' )) break;
			if( *pnt2 == '*' && (pnt2[1] == '/' )) break;
			if( *pnt2 == ' ' || *pnt2 == '\t' || *pnt2 == '\r'
			    || *pnt2 == '\n' || *pnt2 == '\\' )
			{
				*pnt2++;
				continue;
			}
			if( *pnt2 == '(' )
			{
				result = TRUE;
				break;
			}
			/*
			 * Don't know what this is, and we assume it to be
			 * not relevant.
			 */
			break;
		}
		if( result == FALSE )
		{
			/*
			 * We don't care about this.  Continue on.
			 */
			continue;
		}

		/*
		 * Next, figure out whether this is a prototype or
		 * a definition.  Essentially scan forward, keeping track
		 * of parenthesis nesting, and see what happens
		 * afterwards.  There will either be a ';' or a '{'.
		 */
		nesting = 0;
		while(1==1)
		{
			if( *pnt2 == '(' )
			{
				nesting++;
				pnt2++;
				continue;
			}

			if( *pnt2 == ')' )
			{
				nesting--;
				pnt2++;
				if( nesting == 0 )
				{
					break;
				}
				continue;
			}
			if(* pnt2 == '\0' )
			{
				break;
			}
			pnt2++;
		}
		/*
		 * OK, look and see what is next...
		 */
		while(1==1)
		{
			if( *pnt2 == ' ' || *pnt2 == '\t' || *pnt2 == '\r'
			    || *pnt2 == '\n' || *pnt2 == '\\' )
			{
				*pnt2++;
				continue;
			}
			if( *pnt2 == '{' )
			{
				/*
				 * Function body.
				 * We leave these alone.
				 * FIXME - inject new shim function ahead
				 * of real function.
				 */
				if( del != 0)
				{
					pnt += delete_body(pnt, pnt2, del );
				}
				else
				{
					pnt += insert_new_body(pnt, pnt2, funcname,
							       hpnt->lockname,
							       proto_parms,
							       arglist,
							       rtn_valid);
				}
				break;
			}

			if( *pnt2 == ';' )
			{
				/*
				 * Function prototype.
				 * FIXME - inject new prototype after
				 * real prototype.
				 */
				if( del != 0 )
				{
					pnt += delete_proto(pnt, pnt2, funcname, fctname, del);
				}
				else
				{
					insert_new_proto(pnt, pnt2);
				}
				break;
			}
			/*
			 * Neither one. Ignore it.  This happens in comments
			 * in some instances.
			 */
			break;
		}
	}
	return 0;
}

int fix_locks(struct hstruct * hpnt, char * buff, char * lockname, int hdr)
{
	char * pnt;
	char search[256];
	char newstring[1024];
	char ncrstring[256];
	char ncrstring2[256];
	char * stt;

	/*
	 * We have two jobs here.  Rename io_request_lock, and secondly
	 * insert a definition for our own lockname.
	 */
	for(pnt=buff; *pnt; pnt++)
	{
		if( *pnt != 'i' )
		{
			continue;
		}

		if( strncmp(pnt, "io_request_lock", 15) == 0 )
		{
			delete(pnt, pnt + 15);
			inject(pnt, lockname);
		}
	}

	/*
	 * If we are messing with a header, we don't inject a lock definition.
	 */
	if( hdr )
	{
		return 0;
	}

	pnt = strrchr(hpnt->hs_header, '/');
	sprintf(search, "\"%s\"", pnt+1);
	for(pnt=buff; *pnt; pnt++)
	{
		if( *pnt != '"' )
		{
			continue;
		}
		if( strncmp(pnt, search, strlen(search)) == 0 )
		{
			/*
			 * Skip forward to the newline.
			 */
			while(*pnt != '\n') pnt++;
			pnt++;
			if( hpnt->flags & NCR53C9X )
			{
				sprintf(newstring, "#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,1)\nextern spinlock_t %s;\n#else\n#define %s io_request_lock\n#endif\n", lockname, lockname);
			}
			else
			{
				if( hpnt->flags & NCR5380 )
				{
					sprintf(ncrstring, "#define NCR5380_LOCK %s\n", lockname);
					sprintf(ncrstring2, "#define NCR5380_LOCK io_request_lock\n");
				}
				else
				{
					ncrstring[0] = '\0';
					ncrstring2[0] = '\0';
				}
				if( hpnt->flags & NCR53C9X_D )
				{
					stt = "";
				}
				else
				{
					stt = "static ";
				}
				sprintf(newstring, "\n#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,1)\n%sspinlock_t %s = SPIN_LOCK_UNLOCKED;\n%s#else\n#define %s io_request_lock\n%s#endif\n", stt, lockname, ncrstring, lockname, ncrstring2);
			}
			inject(pnt, newstring);
			
			return 0;
		}

	}
	/*
	 * This is normal for ncr5380.c...
	 */
	fprintf(stderr, "Could not find insert spot\n");
	return -1;
}

int process(struct hstruct * hpnt, char * buff, int oldsize, int hdr)
{
	/*
	 * Several independent things we must do here.
	 *
	 * 1) Insert definition for new lock.
	 * 2) Insert prototypes for new shim functions.
	 * 3) Inject new shim functions.
	 * 4) Update template to use new shim functions.
	 *
	 * Some of these are related tasks.  The code to handle
	 * the new shim functions, for example has to search for
	 * specific names in the source file, and then perform
	 * one of several actions depending upon what it was that
	 * we found.
	 */
	process_shim(hpnt, buff, hpnt->eh_strategy_handler, NODELETE,
		     "Scsi_Cmnd * SCpnt", "SCpnt", 1, NULL);
	process_shim(hpnt, buff, hpnt->eh_abort_handler, NODELETE,
		     "Scsi_Cmnd * SCpnt", "SCpnt", 1, NULL);
	process_shim(hpnt, buff, hpnt->eh_device_reset_handler, NODELETE,
		     "Scsi_Cmnd * SCpnt", "SCpnt", 1, NULL);
	process_shim(hpnt, buff, hpnt->eh_bus_reset_handler, NODELETE,
		     "Scsi_Cmnd * SCpnt", "SCpnt", 1, NULL);
	process_shim(hpnt, buff, hpnt->eh_host_reset_handler, NODELETE,
		     "Scsi_Cmnd * SCpnt", "SCpnt", 1, NULL);
	process_shim(hpnt, buff, hpnt->command,
		     strcmp(hpnt->queuecommand, "NULL") != 0,
		     "Scsi_Cmnd * SCpnt", "SCpnt", 1, "command");
	process_shim(hpnt, buff, hpnt->queuecommand, NODELETE,
		     "Scsi_Cmnd * SCpnt, void (*done) (Scsi_Cmnd *)",
		     "SCpnt, done", 
		     (strcmp(hpnt->use_new_eh_code, "TRUE") == 0) ? DELETEIT : NODELETE, NULL);
	if( strcmp(hpnt->use_new_eh_code, "TRUE" ) == 0 )
	{
		process_shim(hpnt, buff, hpnt->detect, NODELETE,
			     "Scsi_Host_Template * tpnt", "tpnt", 1, NULL);
	}

	/*
	 * Comment out the old error handling functions.
	 */
	if( strcmp(hpnt->abort, "NULL") != 0 )
	{
		process_shim(hpnt, buff, hpnt->abort, COMMENTIT,
			     "Scsi_Cmnd * SCpnt", "SCpnt", 1, "abort");
	}

	if( strcmp(hpnt->reset, "NULL") != 0 )
	{
		process_shim(hpnt, buff, hpnt->reset, COMMENTIT,
			     "Scsi_Cmnd * SCpnt", "SCpnt", 1, "reset");
	}

	fix_locks(hpnt, buff, hpnt->lockname, hdr);

	/*
	 * Return the length of the new buffer.
	 */
	return strlen(buff);
}

int fixup(struct hstruct * hpnt, char * filename, int ishdr)
{
	int fd;
	int ret;
	int newsize;
	struct stat st;
	char newfile[MAX_PATH];
	char * buff;
	printf("\tFixing %s...\n", filename);

	/*
	 * First open the file, figure out how large it is, and pull
	 * it into a buffer.
	 */
	fd = open(filename, O_RDONLY | O_BINARY);
	if( fd == -1 )
	{
		fprintf(stderr, "Unable to open %s\n", filename);
		return -1;
	}
	
	ret = fstat(fd, &st);
	if( ret == -1 )
	{
		fprintf(stderr, "Unable to stat %s\n", filename);
		return -1;
	}

	buff = (char *) malloc(st.st_size + 4096);
	memset(buff, 0, st.st_size + 4096);
	ret = read(fd, buff, st.st_size);
	if( ret != st.st_size )
	{
		fprintf(stderr, "Unable to read %s\n", filename);
		return -1;
	}

	if( strlen(buff) != st.st_size )
	{
		fprintf(stderr, "Unable to do strlen %s\n", filename);
		return -1;
	}

	close(fd);

	/*
	 * Process the buffer here.
	 */
	newsize = process(hpnt, buff, st.st_size, ishdr);

	/*
	 * Now write out the modified file...
	 */
	sprintf(newfile, "%s.new", filename);
	unlink(newfile);
	fd = open(newfile, O_RDWR | O_CREAT | O_BINARY, 0666);
	if( fd == -1 )
	{
		fprintf(stderr, "Unable to open %s\n", newfile);
		return -1;
	}

	ret = write(fd, buff, newsize);
	if( ret != newsize )
	{
		fprintf(stderr, "Unable to write %s\n", newfile);
		return -1;
	}
	close(fd);

	free(buff);
	return 0;
}

main(int argc, char * argv[])
{
  struct hstruct * hpnt;
  int i;

  printf("Limit is %d\n", sizeof(drivers) / sizeof(struct hstruct));
  for(i=0, hpnt = &drivers[0]; 
      i < sizeof(drivers) / sizeof(struct hstruct); 
      hpnt = &drivers[++i])
    {
	    if( argc > 1 && strcmp(argv[1], hpnt->hs_name) != 0 )
	    {
		    continue;
	    }
	    if( fixup(hpnt, hpnt->hs_name, FALSE) == -1 )
	    {
		    fprintf(stderr, "Fixup of %s failed\n", hpnt->hs_name);
	    }
	    if( fixup(hpnt, hpnt->hs_header, TRUE) == -1 )
	    {
		    fprintf(stderr, "Fixup of %s failed\n", hpnt->hs_header);
	    }
    }

}
